@@ -56,74 +56,79 @@
 #' @importFrom grid unit
 #' @keywords hplot
 plot_base <- setRefClass(
-
+  
   Class = "plot_base",
-
+  
   fields = c("top", "alternateFrame", "rmlist", "codes", "mode"),
-
+  
   methods = list(
-
+    
     #' Plot Windows
     plotWindow = function() {
-
+      
       # note: The initializeDialog() generates "top"
       initializeDialog(window = topwindow, title = getWindowTitle())
       top            <<- topwindow
       alternateFrame <<- tkframe(top)
-
+      
       setFront()
-
+      
       # OK
       onOK <- function() {
-
+        
         # doItAndPrint mode
         mode <<- 1
+        codes <<- ""
         parms <- getParms()
-
+        
         closeDialog()
-
+        
         errorCode <- checkError(parms)
         if (errorCode == TRUE) {
           removeRmlist()
           return()
         } else if (errorCode == FALSE) {
-
-          logger("require(\"ggplot2\")")
-
+          
+          commandDoIt("require(\"ggplot2\")", log = TRUE)
+          
           setDataframe(parms)
           
           .plot <- getPlot(parms)
-          logger("print(.plot)")
+          commandDoIt("print(.plot)", log = TRUE)
           response <- tryCatch({
-              print(.plot)
-              ""
-            }, error = function(ex) {
-              tclvalue(RcmdrTkmessageBox(
-                message = getMessage(),
-                title   = gettextKmg2("Error"),
-                icon    = "error",
-                type    = "ok",
-                default = "ok"
-              ))
-            }
+            print(.plot)
+            ""
+          }, error = function(ex) {
+            tclvalue(RcmdrTkmessageBox(
+              message = getMessage(),
+              title   = gettextKmg2("Error"),
+              icon    = "error",
+              type    = "ok",
+              default = "ok"
+            ))
+          }
           )
           if (response == "ok") {
             removeRmlist()
             return()
           }
           if (parms$save == "1") savePlot(.plot)
+          
+          pos <- 1
+          assign(".lastcom", paste0(codes, "\n"), envir = as.environment(pos))
+          
         }
         
         removeRmlist()
-
+        
         activateMenus()
         tkfocus(CommanderWindow())
-
+        
       }
-
+      
       setBack()
       
-
+      
       # note: The OKCancelHelp() generates "buttonsFrame"
       OKCancelHelp(window = top, helpSubject = getHelp())
       
@@ -132,9 +137,9 @@ plot_base <- setRefClass(
         
         # justDoIt mode
         mode <<- 0
-
+        codes <<- ""
         parms <- getParms()
-
+        
         errorCode <- checkError(parms)
         if (errorCode == TRUE) {
           removeRmlist()
@@ -163,26 +168,25 @@ plot_base <- setRefClass(
           }
         }
         removeRmlist()
-        
       }
       previewButton <- buttonRcmdr(
         rightButtonsBox, text = gettextKmg2("Preview"), foreground = "yellow",
         width = nchar(gettextKmg2("Preview")), command = onPreview, 
         image = "::image::applyIcon", compound = "left"
-        )
-
+      )
+      
       tkgrid(previewButton, row = 0, column = 3, sticky = "nw")
       tkgrid.configure(previewButton, padx = c(5, 0))
       tkgrid(buttonsFrame, sticky = "nw")
       dialogSuffix()
-
+      
       return()
-
+      
     },
-
+    
     #' Save Plot
-    savePlot = function(plot) {
-
+    savePlot = function(plot, useGgsave = TRUE) {
+      
       plotName <- deparse(substitute(plot))
       if (.Platform$OS.type == "windows") {
         file <- tclvalue(tkgetSaveFile(
@@ -207,18 +211,18 @@ plot_base <- setRefClass(
         ))
       }
       if (file == "") return()
-
-      if (class(.self)[1] == "gkm") {
-        command <- paste0("RcmdrPlugin.KMggplot2::ggsaveKmg2(filename = \"", file, "\", plot = ", plotName, ")")
-      } else {
+      
+      if (useGgsave) {
         command <- paste0("ggsave(filename = \"", file, "\", plot = ", plotName, ")")
+      } else {
+        command <- paste0("RcmdrPlugin.KMggplot2::ggsaveKmg2(filename = \"", file, "\", plot = ", plotName, ")")
       }
-      doItAndPrint(command)
-
+      commandDoIt(command)
+      
       return()
-
+      
     },
-
+    
     #' Register \code{rm()} List
     registRmlist = function(object) {
       
@@ -226,15 +230,15 @@ plot_base <- setRefClass(
       if (class(rmlist) == "uninitializedField") {
         rmlist <<- list(txtObjects)
       } else {
-        rmlist <<- c(rmlist, txtObjects)
+        rmlist <<- unique(c(rmlist, txtObjects))
       }
       return()
-
+      
     },
-
+    
     #' Remove \code{rm()} List
     removeRmlist = function() {
-
+      
       if (class(rmlist) != "uninitializedField") {
         command <- do.call(paste, c(rmlist, sep=", "))
         command <- paste0("rm(", command, ")")
@@ -242,65 +246,65 @@ plot_base <- setRefClass(
         rmlist <<- list()
       }
       return()
-
+      
     },
-
+    
     #' Set Front
     setFront = function() {
-
+      
       return()
-
+      
     },
-
+    
     #' Set Back
     setBack = function() {
-
+      
       return()
-
+      
     },
-
+    
     #' Get Window Title
     getWindowTitle = function() {
-
+      
       "plot_base"
-
+      
     },
-
+    
     #' Get Help
     getHelp = function() {
       
       "plot_base"
       
     },
-
+    
     #' Get Parameters
     getParms = function() {
-
+      
       x      <- "x"
       y      <- "y"
       z      <- "z"
       s      <- "s"
       t      <- "t"
-
+      
       x      <- checkVariable(x)
       y      <- checkVariable(y)
       z      <- checkVariable(z)
       s      <- checkVariable(s)
       t      <- checkVariable(t)
-
+      
       xlab   <- ""
       xauto  <- x
       ylab   <- ""
       yauto  <- y
       zlab   <- ""
       main   <- ""
-
-      size   <- "16"
+      
+      size   <- "14"
       family <- "0"
       colour <- "0"
       save   <- "0"
       theme  <- "0"
-
+      
       options(
         kmg2FontSize   = size,
         kmg2FontFamily = family,
@@ -308,26 +312,26 @@ plot_base <- setRefClass(
         kmg2SaveGraph  = save,
         kmg2Theme      = theme
       )
-
+      
       list(
         x = x, y = y, z = z, s = s, t = t,
         xlab = xlab, xauto = xauto, ylab = ylab, yauto = yauto, zlab = zlab, main = main,
         size = size, family = family, colour = colour, save = save, theme = theme
       )
-
+      
     },
-
+    
     #' Check themes.
     checkTheme = function(index) {
-
+      
       if (index == "theme_bw") {
         theme <- "theme_bw"
       } else if (index == "theme_simple") {
         theme <- "RcmdrPlugin.KMggplot2::theme_simple"
       } else if (index == "theme_classic") {
         theme <- "theme_classic"
-      } else if (index == "theme_gray") {
-        theme <- "theme_gray"
+      } else if (index == "theme_grey") {
+        theme <- "theme_grey"
       } else if (index == "theme_minimal") {
         theme <- "theme_minimal"
       } else if (index == "theme_linedraw") {
@@ -358,18 +362,22 @@ plot_base <- setRefClass(
         theme <- "ggthemes::theme_fivethirtyeight"
         commandDoIt("ggthemes_data <- ggthemes::ggthemes_data")
         registRmlist(ggthemes_data)
+      } else if (index == "theme_foundation") {
+        theme <- "ggthemes::theme_foundation"
       } else if (index == "theme_gdocs") {
         theme <- "ggthemes::theme_gdocs"
       } else if (index == "theme_hc") {
         theme <- "ggthemes::theme_hc"
         commandDoIt("ggthemes_data <- ggthemes::ggthemes_data")
         registRmlist(ggthemes_data)
+      } else if (index == "theme_igray") {
+        theme <- "ggthemes::theme_igray"
+      } else if (index == "theme_pander") {
+        theme <- "ggthemes::theme_pander"
       } else if (index == "theme_par") {
         theme <- "ggthemes::theme_par"
         commandDoIt("ggthemes_data <- ggthemes::ggthemes_data")
         registRmlist(ggthemes_data)
-      } else if (index == "theme_pander") {
-        theme <- "ggthemes::theme_pander"
       } else if (index == "theme_solarized") {
         theme <- "ggthemes::theme_solarized"
         commandDoIt("ggthemes_data <- ggthemes::ggthemes_data")
@@ -384,15 +392,13 @@ plot_base <- setRefClass(
         theme <- "RcmdrPlugin.KMggplot2::theme_wsj2"
         commandDoIt("ggthemes_data <- ggthemes::ggthemes_data")
         registRmlist(ggthemes_data)
-      } else if (index == "theme_igray") {
-        theme <- "ggthemes::theme_igray"
       } else {
         theme <- "theme_bw"
       }
       theme
       
     },
-
+    
     #' Check variable length.
     checkVariable = function(var) {
       
@@ -402,14 +408,14 @@ plot_base <- setRefClass(
       var
       
     },
-
+    
     #' Check Error
     checkError = function(parms) {
-
+      
       errorCode <- FALSE
-
+      
     },
-
+    
     #' Set \code{data.frame}
     setDataframe = function(parms) {
       
@@ -431,16 +437,16 @@ plot_base <- setRefClass(
       }
       command <- do.call(paste, c(var, list(sep = ", ")))
       command <- paste0(".df <- data.frame(", command, ")")
-
+      
       commandDoIt(command)
       registRmlist(.df)
-
+      
     },
     
     #' Get Ggplot
     getGgplot = function(parms) {
       
-      "ggplot(data.frame(1), aes(x = 1, y = 1)) + "
+      "ggplot(data.frame(1), aes(x = 1, y = 1)) + \n  "
       
     },
     
@@ -454,7 +460,7 @@ plot_base <- setRefClass(
     #' Get Scale
     getScale = function(parms) {
       
-      "scale_y_continuous(expand = c(0.01, 0)) + "
+      "scale_y_continuous(expand = c(0.01, 0)) + \n  "
       
     },
     
@@ -464,75 +470,75 @@ plot_base <- setRefClass(
       ""
       
     },
-
+    
     #' Get Facet
     getFacet = function(parms) {
-
+      
       if (length(parms$s) != 0 && length(parms$t) != 0) {
-        facet <- "facet_grid(s ~ t) + "
+        facet <- "facet_grid(s ~ t) + \n  "
       } else if (length(parms$s) != 0) {
-        facet <- "facet_wrap( ~ s) + "
+        facet <- "facet_wrap( ~ s) + \n  "
       } else if (length(parms$t) != 0) {
-        facet <- "facet_wrap( ~ t) + "
+        facet <- "facet_wrap( ~ t) + \n  "
       } else {
         facet <- ""
       }
       facet
-
+      
     },
     
     #' Get Xlab
     getXlab = function(parms) {
-
+      
       if (nchar(parms$xlab) == 0) {
-        xlab <- "xlab(NULL) + "
+        xlab <- "xlab(NULL) + \n  "
       } else if (parms$xlab == "<auto>") {
-        xlab <- paste0("xlab(\"", parms$xauto, "\") + ")
+        xlab <- paste0("xlab(\"", parms$xauto, "\") + \n  ")
       } else {
-        xlab <- paste0("xlab(\"", parms$xlab, "\") + ")
+        xlab <- paste0("xlab(\"", parms$xlab, "\") + \n  ")
       }
       xlab
-
+      
     },
     
     #' Get Ylab
     getYlab = function(parms) {
-
+      
       if (nchar(parms$ylab) == 0) {
-        ylab <- "ylab(NULL) + "
+        ylab <- "ylab(NULL) + \n  "
       } else if (parms$ylab == "<auto>") {
-        ylab <- paste0("ylab(\"", parms$yauto, "\") + ")
+        ylab <- paste0("ylab(\"", parms$yauto, "\") + \n  ")
       } else {
-        ylab <- paste0("ylab(\"", parms$ylab, "\") + ")
+        ylab <- paste0("ylab(\"", parms$ylab, "\") + \n  ")
       }
       ylab
-
+      
     },
     
     #' Get Zlab
     getZlab = function(parms) {
-
+      
       ""
-
+      
     },
-
+    
     #' Get Main
     getMain = function(parms) {
-
+      
       if (nchar(parms$main) == 0) {
         main <- ""
       } else {
-        main <- paste0("labs(title = \"", parms$main, "\") + ")
+        main <- paste0("labs(title = \"", parms$main, "\") + \n  ")
       }
       main
-
+      
     },
     
     #' Get Theme
     getTheme = function(parms) {
-
+      
       paste0(parms$theme, "(base_size = ", parms$size, ", base_family = \"", parms$family, "\")")
-
+      
     },
     
     #' Get Opts
@@ -540,12 +546,12 @@ plot_base <- setRefClass(
       
       opts <- list()
       if (length(parms$s) != 0 || length(parms$t) != 0) {
-        opts <- c(opts, "panel.margin = grid::unit(0.3, \"lines\")")
+        opts <- c(opts, "panel.spacing = grid::unit(0.3, \"lines\")")
       }
-
+      
       if (length(opts) != 0) {
         opts <- do.call(paste, c(opts, list(sep = ", ")))
-        opts <- paste0(" + theme(", opts, ")")
+        opts <- paste0(" + \n  theme(", opts, ")\n")
       } else {
         opts <- ""
       }
@@ -555,7 +561,7 @@ plot_base <- setRefClass(
     
     #' Get Plot
     getPlot = function(parms) {
-
+      
       gg    <- getGgplot(parms)
       geom  <- getGeom(parms)
       scale <- getScale(parms)
@@ -567,7 +573,7 @@ plot_base <- setRefClass(
       main  <- getMain(parms)
       theme <- getTheme(parms)
       opts  <- getOpts(parms)
-
+      
       command <- paste0(
         ".plot <- ",
         gg, geom, scale, coord, facet,
@@ -576,28 +582,35 @@ plot_base <- setRefClass(
       commandDoIt(command)
       registRmlist(.plot)
       return(.plot)
-
+      
     },
-
+    
     #' Get Plot Error Message
     getMessage = function() {
-
+      
       gettextKmg2("Plot failed.  Please check the data and variables, or try other options.")
-
+      
     },
     
     #' An wrapper function for command execution
-    commandDoIt = function(command, log = TRUE) {
+    commandDoIt = function(command, log = FALSE) {
       
-      if (mode == 1) {
-        doItAndPrint(command, log = log)
+      if (codes == "") {
+        codes <<- command
+      } else {
+        codes <<- paste0(codes, "\n", command)
+      }
+      if (log) {
+        logger(command)
+      } else if (mode == 1) {
+        doItAndPrint(command, log = TRUE)
       } else {
         justDoIt(command)
       }
       NULL
       
     }
-
+    
   )
 )
 
@@ -622,8 +635,9 @@ NULL
 #'
 #' \code{savePlot} method saves the plot.
 #'
-#' @usage \S4method{savePlot}{plot_base}(plot)
+#' @usage \S4method{savePlot}{plot_base}(plot, useGgsave)
 #' @param plot \code{ggplot} or \code{recordedplot} class object; the plot to save.
+#' @param useGgsave Boolean; use \code{ggsave}.
 #' @family plot
 #'
 #' @name savePlot,plot_base-method
@@ -1091,8 +1105,9 @@ NULL
 #'
 #' \code{commandDoIt} method gets the plot error message.
 #'
-#' @usage \S4method{commandDoIt}{plot_base}(command)
+#' @usage \S4method{commandDoIt}{plot_base}(command, log)
 #' @param command String; command codes.
+#' @param log Boolean; use \code{logger}.
 #' @family plot
 #'
 #' @name commandDoIt,plot_base-method
